-- Frame:SetBackdropColor
-- AddOn initialization

local options = { 
    type='group',
    args = {
        checkInterval = {
            type = 'range',
            name = 'check interval',
            desc = 'Set the interval in which GoodTarget checks the targets target (in seconds).',
            min = 0.01,
            max = 1.0,
            step = 0.01,
            bigStep = 0.02,
            get = "GetCheckInterval",
            set = "SetCheckInterval",
        },
        scale = {
            type = 'range',
            name = 'scale',
            desc = 'Set the scale of the frames.',
            min = 0.1,
            max = 4.0,
            step = 0.01,
            bigStep = 0.1,
            get = "GetScale",
            set = "SetScale",
        },
        width = {
            type = 'range',
            name = 'width',
            desc = 'Set the width of the frames.',
            min = 10,
            max = 500,
            step = 1,
            bigStep = 1,
            get = "GetWidth",
            set = "SetWidth",
        },
        height = {
            type = 'range',
            name = 'height',
            desc = 'Set the height of the frames.',
            min = 5,
            max = 100,
            step = 1,
            bigStep = 1,
            get = "GetHeight",
            set = "SetHeight",
        },
        playSound = {
            type = 'toggle',
            name = 'sound',
            desc = 'Enables or disables sounds.',
            get = "IsSoundOn",
            set = "SetSoundOnOf",
        },
        isLocked = {
            type = 'toggle',
            name = 'lock',
            desc = 'Locks or unlocks the frames.',
            get = "IsLocked",
            set = "SetLocked",
        },
    },
}


GoodTarget = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")
GoodTarget:RegisterChatCommand("/goodtarget", "/gt", options)

GoodTarget:RegisterDB("GoodTargetDB", "GoodTargetDBPC")
GoodTarget:RegisterDefaults("profile", {
    CheckInterval = 0.05,
    PlaySound = true,
    isLocked = false,
    Scale = 1.0,
    Width = 100,
    Height = 20,
} )



function GoodTarget:Update()
    self:UpdateGroupTargets()
    if(UnitIsFriend("player", "playertarget")) then -- only check for hostile targets
        self:SetNoneTarget();
        self.LastTT = nil;
        return;
    end
    
    
    
	if(UnitIsUnit("player", "playertargettarget")) then --targets target is the player        
		self:SetGoodTarget()
    elseif(UnitExists("playertargettarget")) then --targets target is not the player
		self:SetBadTarget();		
	elseif((not UnitExists("playertargettarget"))) then --there is no targets target
		self:SetNoneTarget();		
	end
    
end

function GoodTarget:UpdateGroupTargets ()
	local groupMode					= nil;
	local groupMembers				= 0;
	if (GetNumPartyMembers() > 0) then
		groupMode				= "party";
        groupMembers			= GetNumPartyMembers();		
	end	
    if (GetNumRaidMembers() > 0) then
			groupMode				= "raid";
			groupMembers			= GetNumRaidMembers();
    end
    
	if not groupMode then return end
	
    self.badTargets = {}
    --self:Print("we are a "..groupMode.." with "..groupMembers)
	for i = 1, groupMembers, 1 do
		local unit = groupMode..i
		if (UnitExists(unit.."target")) and
           (UnitExists(unit.."targettarget")) and
           (not UnitIsUnit("playertarget", unit.."target")) and 
		   (not UnitIsFriend(unit.."target", "player")) and
		   (not UnitIsUnit("player", unit.."targettarget")) then
			
            --self:Print("found bad target")
		   	local alreadyNoticed = false
		   	for _, bad in pairs(self.badTargets) do
		   		if UnitIsUnit(unit.."target", groupMode..bad.."target") then
		   			alreadyNoticed = true
                    --self:Print("...already known")
					break
				end
		   	end
			if not alreadyNoticed then
                --self:Print("...new one")
                --local tmp = {}
                --tmp.id = unit.."target"
                --tmp.isActual=true
				tinsert(self.badTargets, i)
			end
		end
	end
	local nr = 1
	for i, bad in pairs(self.badTargets) do
		--if not bad or not bad.isActual then
		--	tremove(self.badTargets, i)
		--else
            --self:Print("showing frame for "..bad.id)
			self:SetBadTargetFrame(nr, bad)
			nr = nr + 1
			--bad.isActual = false
		--end
	end
	for i=nr, 4, 1 do
		self:HideBadFrame(i)
        
	end
end

function GoodTarget:SetBadTargetFrame(frameNr, badID)
	if frameNr > 4 then return end
    --self.badFrames[frameNr]:SetAttribute("target", badID)
    
    --self.badFrames[frameNr].unitID = badID.."target"
    local tmpID = "party"..badID.."target"
    local tmpFrame = badID
    if GetNumRaidMembers() > 0 then
        tmpID = "raid"..badID.."target"
        self.badFrames[frameNr]:SetAttribute("target", tmpID)
        tmpFrame = frameNr
    end
    --local target = UnitName(tmpID)
    local ttarget = UnitName(tmpID.."target")

    self:ShowBadFrame(frameNr)	
    
    --target = string.sub(target, 1, 12)
    --ttarget = string.sub(ttarget, 1, 12)
    self.badFrames[frameNr].target:SetText(ttarget)
    --self.badFrames[frameNr].target2:SetText(UnitName(badID.."target"))
end

function GoodTarget.FrameOnMouseDown(f, button)
    if not f.isLocked and button == "LeftButton" then
		f:StartMoving()
	end
end

function GoodTarget.FrameOnMouseUp(f)
    f:StopMovingOrSizing()
end

function GoodTarget:OnInitialize()
	self.badTargets = {}
	
    local f=CreateFrame("FRAME","GoodTarget_Frame",UIParent);
    self.frame = f
    
    f:SetMovable(true)
    --f:SetResizable(true)
    f:EnableMouse(true)
    
    --f:SetPoint("CENTER",UIParent);
    f:SetBackdrop({
      bgFile="Interface\\Tooltips\\UI-Tooltip-Background", 
      tile=1, tileSize=10, edgeSize=0, 
      insets={left=0, right=0, top=0, bottom=0}
    });
    f:SetBackdropColor(0.0, 0.0, 0.0, 0.0)
    
    f:SetScript("OnMouseUp", GoodTarget.FrameOnMouseUp)
	f:SetScript("OnMouseDown", GoodTarget.FrameOnMouseDown)
    
    f.target = f:CreateFontString("$parentText","ARTWORK","GameFontNormal");
    f.target:SetPoint("CENTER",f)
    
    f.isLocked = self.db.profile.isLocked
    
    --local bfp=CreateFrame("FRAME","GoodTarget_BadFrames",UIParent);
	self.badFrames = {}
	local parentFrame = f
    for i=1, 4, 1 do
    	local bf = CreateFrame("BUTTON", "GoodTarget_BadFrame"..i, UIParent, "SecureUnitButtonTemplate")
		self.badFrames[i] = bf
        bf:SetBackdrop({
            bgFile="Interface\\Tooltips\\UI-Tooltip-Background", 
            tile=true, tileSize=16, edgeSize=0, 
            insets={left=0, right=0, top=0, bottom=0}
        });
        bf:EnableMouse(true)
        bf:RegisterForClicks("LeftButtonUp")
        
        --bf:SetScript("OnMouseUp", GoodTarget.BadFrameOnMouseUp)
		bf:SetPoint("TOPLEFT", parentFrame, "BOTTOMLEFT")
        bf:Show()
		bf.target = bf:CreateFontString("$parentText","ARTWORK","GameFontNormalSmall")
		bf.target:SetPoint("CENTER", bf)
		self:HideBadFrame(i)
        bf:SetAttribute("type", "target")
        bf:SetAttribute("unit", "party"..i.."target")
		parentFrame = bf
    end
    
    self:SetWidth(self.db.profile.Width); 
    self:SetHeight(self.db.profile.Height);
    self:SetScale(self.db.profile.Scale);
end

function GoodTarget:HideBadFrame(nr)
    self.badFrames[nr].target:SetTextColor(1,1,1,0)		
    self.badFrames[nr]:SetBackdropColor(1.0, 0.0, 0.0, 0)
end

function GoodTarget:ShowBadFrame(nr)
    self.badFrames[nr].target:SetTextColor(1,1,1,1)		
    self.badFrames[nr]:SetBackdropColor(1.0, 0.0, 0.0, 1.0)
end
    
function GoodTarget:OnEnable()
    -- Called when the addon is enabled
    self:RegisterEvent("PLAYER_TARGET_CHANGED")
    self:ScheduleRepeatingEvent("CheckTargetTarget", "checkTarget", self.db.profile.CheckInterval)    
    self:RegisterEvent("checkTarget")
    self.frame:Show()
end

function GoodTarget:checkTarget()
    self:Update()
end

function GoodTarget:OnDisable()
    self.frame:Hide()
end

function GoodTarget:IsSoundOn()
    return self.db.profile.PlaySound
end

function GoodTarget:SetSoundOnOf(isOn)
    self.db.profile.PlaySound = isOn
end

function GoodTarget:IsLocked()
    return self.db.profile.isLocked
end

function GoodTarget:SetLocked(isLocked)
    self.db.profile.isLocked = isLocked
    self.frame.isLocked = self.db.profile.isLocked
end

function GoodTarget:GetScale()
    return self.db.profile.Scale
end

function GoodTarget:SetScale(scale)
    self.db.profile.Scale = scale
    self.frame:SetScale(scale)
    for i=1, 4, 1 do
        self.badFrames[i]:SetScale(scale)
    end
end

function GoodTarget:GetWidth()
    return self.db.profile.Width
end

function GoodTarget:SetWidth(width)
    self.db.profile.Width = width
    self.frame:SetWidth(width)
    for i=1, 4, 1 do
        self.badFrames[i]:SetWidth(width)
    end
end

function GoodTarget:GetHeight()
    return self.db.profile.Height
end

function GoodTarget:SetHeight(height)
    self.db.profile.Height = height
    self.frame:SetHeight(height)
    for i=1, 4, 1 do
        self.badFrames[i]:SetHeight(height)
    end
end

function GoodTarget:GetCheckInterval()
    return self.db.profile.CheckInterval
end

function GoodTarget:SetCheckInterval(interval)
    self.db.profile.CheckInterval = interval
    self:CancelScheduledEvent("CheckTargetTarget")
    self:Update()
    self:ScheduleRepeatingEvent("CheckTargetTarget", "checkTarget", self.db.profile.CheckInterval)
end

function GoodTarget:PLAYER_TARGET_CHANGED()
    self:Update();
end

function GoodTarget:SetGoodTarget() -- set GoodTarget frame to good an play good sound
    if(self.LastTT ~= "player") then
        if self.db.profile.PlaySound and self.LastTT then PlaySoundFile("Sound\\interface\\AuctionWindowOpen.wav") end;
        self.frame.target:SetTextColor(1,1,1,1);
        self.frame.target:SetText("Good");
        self.frame:SetBackdropColor(0.0, 1.0, 0.0, 1.0)
        self.LastTT = "player";
    end
end

function GoodTarget:SetBadTarget() -- set GoodTarget frame to bad an play bad sound
    --self:Print("BT")
    if(self.LastTT == "player" or not self.LastTT) then
        if(self.db.profile.PlaySound) then PlaySoundFile("Sound\\Creature\\Cannon\\CannonDeath.wav") end;
    end
    if(self.LastTT ~= UnitName("playertargettarget")) then 
        self.frame.target:SetTextColor(1,1,01,1);
        self.frame:SetBackdropColor(1.0, 0.0, 0.0, 1.0)
        self.frame.target:SetText(UnitName("playertargettarget"));
        self.LastTT = UnitName("playertargettarget");
    end
end

function GoodTarget:SetNoneTarget() -- clear GoodTarget frame
    if(self.LastTT ~= nil) then
        self.frame.target:SetTextColor(0.5,0.5,0.5,1);
        self.frame:SetBackdropColor(0.0, 0.0, 0.0, 0.0)
        self.frame.target:SetText("");
        self.LastTT = nil;
    end
end




